"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Easy use distributed redis lock implementation
 *
 * @class ELock
 */
class ELock {
    /**
     * Creates an instance of ELock.
     * @param {Redis} redis ioredis client
     * @param {IELockOptions} options options
     * @memberof ELock
     */
    constructor(redis, options) {
        this.client = redis;
        this.prefix = options.prefix ? "l:" + options.prefix : "l:" + new Date().getTime();
        this.timeout = options.timeout || 1000;
    }
    /**
     * Acquire a lock (fail will throw a error)
     *
     * @param {string} [key="default"] lock key
     * @param {number} [timeout=this.timeout] lock time (ms)
     * @memberof ELock
     */
    acquire(key = "default", timeout = this.timeout) {
        return this.client
            .set(this.getKey(key), new Date().getTime(), "PX", timeout, "NX")
            .then((val) => {
            if (!val) {
                throw new Error("acquire lock fail");
            }
            return val;
        });
    }
    /**
     * Get a lock (fail will return null)
     *
     * @param {string} [key="default"] lock key
     * @param {number} [timeout=this.timeout] lock time (ms)
     * @memberof ELock
     */
    get(key = "default", timeout = this.timeout) {
        return this.client.set(this.getKey(key), new Date().getTime(), "PX", timeout, "NX");
    }
    /**
     * Release the lock
     *
     * @param {string} [key="default"] lock key
     * @memberof ELock
     */
    release(key = "default") {
        return this.client.del(this.getKey(key));
    }
    /**
     * Check is locked
     *
     * @param {string} [key="default"] lock key
     * @memberof ELock
     */
    locked(key = "default") {
        return this.client.get(this.getKey(key)).then((res) => !!res);
    }
    /**
     * Get the real lock key in redis
     *
     * @private
     * @param {string} key
     * @returns {string}
     * @memberof ELock
     */
    getKey(key) {
        return this.prefix + ":" + key;
    }
}
exports.default = ELock;
